--|
--| Filename        : $Source: /home/erdmann/ASCL/Library/RCS/ascl-sv-agent.adb,v $
--| Description     : Template Specification
--| Author          : Michael Erdmann
--| Created On      : 25.3.1999
--| Last Modified By: $Author: erdmann $
--| Last Modified On: $Date: 1999/09/26 18:17:21 $
--| Status          : $State: Exp $
--|
--|
--| Functional Description
--| ======================
--| This package maintains a per agent output list.
--|
--| Component Data
--| ==============
--|
--|
--| Operations on the component data
--| ================================
--|
--| Error Handling
--| ==============
--|
--|
--| Extension
--| =========
--|
--| Restrictions
--| ============
--| Tasking: yes/no
--| Y2K    :
--|
--|
--| History
--| =======
--| $Log: ascl-sv-agent.adb,v $
--| Revision 1.1  1999/09/26 18:17:21  erdmann
--| Put/Get added and Process interface changed
--|
--| Revision 1.1  1999/08/06 17:00:21  erdmann
--| No comments
--|
--|
--|
package body ASCL.SV.Agent is

   Version : constant String := "$Id: ascl-sv-agent.adb,v 1.1 1999/09/26 18:17:21 erdmann Exp erdmann $";

   ---====================================================================---
   ---===                O B J  E C T     D A T A                      ===---
   ---====================================================================---

   type Packet;
   type Packet_Access is access Packet;

   type Packet( Packet_Size : Positive ) is record
         next : Packet_Access := null;
         data : String( 1..Packet_Size );
      end record;

   --|
   --| This is the instance of the component internal data.
   --|
   type Object_Data is record
         head : Packet_Access := null;
         tail : Packet_Access := null;
         free : Packet_Access := null;
      end record;

   ---=====================================================================---
   ---===         L O C A L   S U P P O R T   P R O C E D U R E S       ===---
   ---=====================================================================---
   function Initialize return Object_Data_Access is
   begin
      return new Object_Data;
   end Initialize;


   ---======================================================================---
   ---===             C O M P O  N E N T    I N T E R F A C E            ===---
   ---======================================================================---


   ---=====================================================================---
   ---===           A T T R I B U T E    F U N C T I O N S              ===---
   ---=====================================================================---

   --|
   --| Check if there is data in the agents output list.
   --|
   function Data_Available( this : in Object'Class ) return Boolean is
      data : Object_Data_Access := this.data;
   begin
      if data = null then
         raise Not_Initialized;
      end if;

      return  data.head /= null;

   end Data_Available;

   ---=====================================================================---
   ---===                        M E T H O D S                          ===---
   ---=====================================================================---

   --|
   --| Purpose:
   --|    This procedure is used by the Agent process to send packages
   --|    back to the client.
   --|
   --| Pre-Conditions:
   --|
   --| Post-Conditions:
   --|
   --|
   procedure Put( this   : in out Object'Class;
                  toSend : in String ) is
      data : Object_Data_Access := this.data;
      pkt  : Packet_Access;
   begin
      if data = null then
         raise Not_Initialized;
      end if;

      pkt := new Packet( toSend'Length );
      pkt.Data := toSend;

      if data.head = null then
         data.head := pkt;
      else
         data.tail.next := pkt;
      end if;

      data.tail := pkt;

   end Put;

   --|
   --| Purpose:
   --|
   --| Pre-Conditions:
   --|
   --| Post-Conditions:
   --|    The package is placed in the free list.
   --|
   function Get( this : in Object'Class ) return String is
      data   : Object_Data_Access := this.data;
      result : Packet_Access      := null;
   begin
      if data = null then
         raise Not_Initialized;
      end if;

      if data.head /= null then
         result := data.head;
         data.head := data.head.next;
      else
         raise Usage_Error;
      end if;

      result.next := data.free;
      data.free   := result;

      return result.Data;
   end Get;


end ASCL.SV.Agent;
