--                              -*- Mode: Ada -*-
-- Filename        : $Source: /home/erdmann/ASCL/Library/RCS/ascl-sv-server.ads,v $
-- Description     : server component
-- Author          : Michael Erdmann
-- Created On      : Sat Jul 25 19:11:10 1998
-- Last Modified By: $Author: erdmann $
-- Last Modified On: $Date: 1999/09/28 07:40:22 $
-- Status          : $State: Exp $
--
-- Functional Description
-- ======================
-- This is a server object. The server waits for scoket connections 
-- on a specified port. The port number may by given as the second
-- parameter of the initialize call. The server will create instances
-- of the Agent-Datatype for every client/server relation by means of 
-- the create method. As a result a server instance supports only
-- a single agent type.
--
--
-- History
-- =======
-- $Log: ascl-sv-server.ads,v $
-- Revision 1.3  1999/09/28 07:40:22  erdmann
-- Debuging Support added
--
-- Revision 1.2  1999/09/26 18:18:53  erdmann
-- Agent interface changed (Put/Get)
--
-- Revision 1.1  1999/09/25 19:37:32  erdmann
-- No comments
--
with ASCL.SV.Agent;   
with ASCL.Debugging_Support;   use ASCL;

package ASCL.SV.Server is

   type Object( Port : Positive ) is abstract tagged private;
   type Reference is access all Object'Class; 	        	

   --=======================================================================--
   --===                    COMPONENT INTERFACE                          ===--
   --=======================================================================--
   procedure Initialize( O     : in Reference; 
                         Port  : in Integer := 0;
                         debug : in Debugging_Support.Handle := null );

   procedure Finalize( O : in Reference );

   --=======================================================================--
   --===                        EXTENSIONS                               ===--
   --=======================================================================--

   function  Create( O : in Object ) return Agent.Reference is abstract;
   procedure Delete( A : Agent.Object'Class ) is abstract;   

private

   type Object_Data;
   type Object_Data_Access is access Object_Data;

   function Initialize( Port : in Positive ) return Object_Data_Access;

   type Object( Port : Positive ) is abstract tagged record
         Data  : Object_Data_Access := Initialize( Port );
      end record;

end ASCL.SV.Server;



